// ===================================================================
//
// Noc_Braitenberg_1
// CBP_FBO
// 22-12-21 Neural Network implemetation of Braitenberg
//
// ==================================================================

#include <webots/robot.h>
#include <webots/motor.h> 
#include <stdio.h>
#include <webots/light_sensor.h>

#include "CBP_Helper_3.h"

#define TIME_STEP 32
#define MAX_SPEED 6.28
#define PI 3.14159265358979
WbDeviceTag left_motor, right_motor;
WbDeviceTag ls0, ls7;

int main(int argc, char **argv) {

  double omegaL, omegaR;
  double leftEyeVal, rightEyeVal;
  double leftIntens, rightIntens;
  
  // Variables for Neural Circuit ==========================
  // Neurons - Set Initial Values here ---------------------
  double u1 = 0;
  double u2 = 0;
  double u3 = 0;
  double u4 = 0;
  // Neurons - Set time constant-----------------------------
  double tau = 0.05;
  // Neurons - Internal variables for dynamics   ------------     
  double du1dt, du2dt, du3dt,du4dt;
  double dT = 0.01;
  // ========================================================
  
  // Webots and CBP_FBO Engine Initialize -------------------
  wb_robot_init();
  deviceInit();

  // Main Computational loop --------------------------------
  while (wb_robot_step(TIME_STEP) != -1) {
    
    // Webots read sensors
    leftEyeVal = wb_light_sensor_get_value(ls7);    
    rightEyeVal = wb_light_sensor_get_value(ls0);    

    // Normalize sensor values to range 0.0 - 1.0
    leftIntens  = getLightVal(leftEyeVal);
    rightIntens = getLightVal(rightEyeVal);
    
    // Calculate RHS of neurons -----------------------------
    // First the sensor neurons --------------------------  
    //du1dt = ;  
    //du2dt = ;
    // Now the motor neurons - cross connexion to 
    // move towards the light -----------------------------
    //du3dt = ;  
    //du4dt = ;

    // Update the Neuron values (Euler Integrator)---------
    u1 += du1dt*dT;
    u2 += du2dt*dT;
    u3 += du3dt*dT;
    u4 += du4dt*dT;
            
    printf("u1 = %f, u2 = %f, u3 = %f, u4 = %f\n",
    u1,u2,u3,u4);

    // send motor neuron values to motors-------------------
    omegaL = u3*MAX_SPEED;
    omegaR = u4*MAX_SPEED;   
    // Webots stuff
    wb_motor_set_velocity(left_motor,  omegaL);
    wb_motor_set_velocity(right_motor, omegaR);

  };

  // Webots stuff 
  wb_robot_cleanup();
  return 0;
}

// Function to initialize Webots simulator ------------------
void deviceInit() {
 // get a handler to the motors and set target position to infinity (speed control).
  left_motor = wb_robot_get_device("left wheel motor");
  right_motor = wb_robot_get_device("right wheel motor");
  wb_motor_set_position(left_motor, INFINITY);
  wb_motor_set_position(right_motor, INFINITY);
  wb_motor_set_velocity(left_motor, 0.0);
  wb_motor_set_velocity(right_motor, 0.0);

  ls0 = wb_robot_get_device("ls0");
  wb_light_sensor_enable(ls0, TIME_STEP); 
  ls7 = wb_robot_get_device("ls7");
  wb_light_sensor_enable(ls7, TIME_STEP);
       
  } 